package ut.maxorato.bsc.crawler.analysis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ut.maxorato.bsc.crawler.model.AnalysisContext;
import ut.maxorato.bsc.crawler.model.PathElement;
import ut.maxorato.bsc.crawler.model.PathElementType;
import ut.maxorato.bsc.crawler.model.UrlTemplate;
import ut.maxorato.bsc.crawler.model.UrlTemplateGroup;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

public class TemplateFinalizer implements Runnable {
    private static final Logger log = LoggerFactory.getLogger(TemplateFinalizer.class);

    private final AnalysisContext context;
    private final boolean writeResults;

    public TemplateFinalizer(AnalysisContext context, boolean writeResults) {
        this.context = context;
        this.writeResults = writeResults;
    }

    private void processTemplate(UrlTemplate urlTemplate) {
        int selectedElementIndex = -1;
        int highestUniqueValueCount = 0;
        int index = 0;
        int numericElementCount = 0;

        for (PathElement element : urlTemplate.elements) {
            if (element.type == PathElementType.NUM_VALUE) {
                int uniqueValueCount = new HashSet<>(element.values).size();

                if (uniqueValueCount > highestUniqueValueCount) {
                    selectedElementIndex = index;
                    highestUniqueValueCount = uniqueValueCount;
                }

                numericElementCount++;
            }

            index++;
        }

        if (highestUniqueValueCount < 10) {
            return;
        } else if (numericElementCount == 1) {
            context.finalTemplates.put(urlTemplate.key, urlTemplate);
            return;
        }

        PathElement selectedElement = urlTemplate.elements.get(selectedElementIndex);

        for (int i = 0; i < urlTemplate.valueSetCount; i++) {
            String urlKey = urlTemplate.getKeyWithAllConstantExcept(i, selectedElementIndex);
            UrlTemplate finalTemplate = context.finalTemplates.get(urlKey);

            if (finalTemplate == null) {
                finalTemplate = new UrlTemplate(PathElement.buildPathSingleVariableNumber(urlTemplate.elements, i, selectedElementIndex));
                context.finalTemplates.put(urlKey, finalTemplate);
            } else {
                finalTemplate.elements.get(selectedElementIndex).values.add(selectedElement.values.get(i));
                finalTemplate.valueSetCount++;
            }
        }
    }

    @Override
    public void run() {
        for (UrlTemplateGroup group : context.templateGroups.values()) {
            for (UrlTemplate template : group.templates.values()) {
                processTemplate(template);
            }
        }

        log.info("Final template count is {}", context.finalTemplates.size());

        for (UrlTemplate template : context.finalTemplates.values()) {
            int count = 0;

            for (PathElement element : template.elements) {
                if (element.type == PathElementType.NUM_VALUE) {
                    Set<Integer> values = new HashSet<>();
                    for (String value : element.values) {
                        values.add(Integer.parseInt(value));
                    }
                    template.finalValues = new ArrayList<>(values);
                    template.urlWithPlaceholder = template.nthUrlWithPlaceholder(template.valueSetCount - 1, count);
                    template.knownValue = Integer.parseInt(element.values.get(template.valueSetCount - 1));
                    Collections.sort(template.finalValues);
                    break;
                }

                count++;
            }

            if (template.valueSetCount >= 10) {
                log.info("Template ({} values): {}", template.valueSetCount, template.key);
            }
        }

        if (!writeResults) {
            context.report.afterSingleNumericTemplateCount = context.finalTemplates.size();
        }
    }
}
