package ut.maxorato.bsc.crawler.analysis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ut.maxorato.bsc.crawler.model.AnalysisContext;
import ut.maxorato.bsc.crawler.model.UrlTemplate;
import ut.maxorato.bsc.crawler.model.UrlTemplateGroup;

import java.util.Map;

public class TemplateGroupProducer implements Runnable {
    private static final Logger log = LoggerFactory.getLogger(TemplateGroupProducer.class);

    private final AnalysisContext context;

    public TemplateGroupProducer(AnalysisContext context) {
        this.context = context;
    }

    @Override
    public void run() {
        for (Map.Entry<String, UrlTemplate> entry : context.initialTemplates.entrySet()) {
            UrlTemplate template = entry.getValue();

            String groupKey = template.getGroupKey();
            UrlTemplateGroup group = context.templateGroups.get(groupKey);
            if (group == null) {
                group = new UrlTemplateGroup(groupKey);
                context.templateGroups.put(groupKey, group);
            }

            group.templates.put(template.key, template);
        }

        context.report.initialGroupCount = context.templateGroups.size();

        log.info("Initial templates organized into {} groups.", context.templateGroups.size());
    }
}
