package ut.maxorato.bsc.crawler.diff;

import ut.maxorato.bsc.crawler.model.RequestContext;

public class RemovedCategoryComparisonStrategy extends PageComparisonStrategy {
    private final String secondEntityUrl;
    private QueryResult secondEntityResult;
    private DomDelta firstVsSecond;

    public RemovedCategoryComparisonStrategy(RequestContext requestContext, String firstEntityUrl, String secondEntityUrl) {
        super(requestContext, firstEntityUrl);
        this.secondEntityUrl = secondEntityUrl;
    }

    protected void loadSecondEntityResult() {
        if (secondEntityResult == null) {
            secondEntityResult = httpQueryExecutor.execute(secondEntityUrl);
        }
    }

    protected void loadEntityPageDelta() {
        if (firstVsSecond == null) {
            firstVsSecond = domDeltaCalculator.calculate(originalResult.text, secondEntityResult.text);
            firstVsSecond = filterEffectiveDelta(firstVsSecond);
        }
    }

    public boolean isValidAlternateUrl(String alternateFirstEntityUrl) {
        ComparisonContext context = new ComparisonContext(alternateFirstEntityUrl);

        Boolean initialResult = responseCodeBasedComparison(context);
        if (initialResult != null) {
            return initialResult;
        }

        loadSecondEntityResult();

        // No basis to compare with in this case
        if (!secondEntityResult.isSuccess()) {
            return false;
        }

        loadEntityPageDelta();
        initialiseContextResult(context);

        DomDelta originalVsAlternate = domDeltaCalculator.calculate(originalResult.text, context.alternateResult.text);
        originalVsAlternate = filterEffectiveDelta(originalVsAlternate);

        if (originalVsAlternate.differences.size() == 0 && firstVsSecond.differences.size() > 0) {
            return true;
        } else if (firstVsSecond.differences.size() > 0) {
            DomDelta prefixOne = removePrefixes(firstVsSecond, originalVsAlternate);
            DomDelta prefixTwo = removePrefixes(originalVsAlternate, firstVsSecond);

            if (prefixOne.differences.size() == prefixTwo.differences.size() && prefixOne.differences.size() == firstVsSecond.differences.size()) {
                return true;
            } else if (firstVsSecond.getMatchCount(originalVsAlternate) == 0) {
                return true;
            }
        }

        return false;
    }
}
