package ut.maxorato.bsc.crawler.diff;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ut.maxorato.bsc.crawler.model.RequestContext;

public class RemovedFixComparisonStrategy extends PageComparisonStrategy {
    private static final Logger log = LoggerFactory.getLogger(RemovedFixComparisonStrategy.class);

    protected final String emptyPageUrl;
    protected QueryResult emptyResult;
    protected DomDelta originalVsEmpty;

    public RemovedFixComparisonStrategy(RequestContext requestContext, String originalUrl, String emptyPageUrl) {
        super(requestContext, originalUrl);
        this.emptyPageUrl = emptyPageUrl;
    }

    protected void loadEmptyResult() {
        if (emptyResult == null) {
            emptyResult = httpQueryExecutor.execute(emptyPageUrl);
        }
    }

    protected void loadEmptyPageDelta() {
        if (originalVsEmpty == null) {
            originalVsEmpty = domDeltaCalculator.calculate(originalResult.text, emptyResult.text);
            originalVsEmpty = filterEffectiveDelta(originalVsEmpty);
        }
    }

    public boolean isValidAlternateUrl(String alternateUrl) {
        ComparisonContext context = new ComparisonContext(alternateUrl);

        Boolean initialResult = responseCodeBasedComparison(context);
        if (initialResult != null) {
            return initialResult;
        }

        loadEmptyResult();

        // If empty result means error code, then this is not empty result
        if (!emptyResult.isSuccess()) {
            return true;
        }

        loadEmptyPageDelta();
        initialiseContextResult(context);

        DomDelta originalVsAlternate = domDeltaCalculator.calculate(originalResult.text, context.alternateResult.text);
        originalVsAlternate = filterEffectiveDelta(originalVsAlternate);

        if (originalVsAlternate.differences.size() == 0 && originalVsEmpty.differences.size() > 0) {
            return true;
        }

        return false;
    }
}
