package ut.maxorato.bsc.crawler.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class PathElement {
    public final PathElementType type;
    public final boolean isGetPart;
    public final List<String> values;

    public PathElement(PathElementType type, boolean isGetPart, String value) {
        this(type, isGetPart);

        values.add(value);
    }

    public PathElement(PathElementType type, boolean isGetPart, List<String> values) {
        this.type = type;
        this.isGetPart = isGetPart;
        this.values = values;
    }

    public PathElement(PathElementType type, boolean isGetPart) {
        this.type = type;
        this.isGetPart = isGetPart;
        this.values = new ArrayList<>();
    }

    public String keyPart() {
        switch (type) {
            case DOMAIN:
                return values.get(0);
            case KEY:
                return values.get(0);
            case NUM_VALUE:
                return "{N}";
            case TEXT_VALUE:
                return "{T}";
            case TEXT_STATIC:
                return values.get(0);
            case SEPARATOR:
                return values.get(0);
            default:
                throw new IllegalStateException();
        }
    }

    public String constantKeyPart(int index, boolean evenTextValue) {
        switch (type) {
            case DOMAIN:
                return values.get(0);
            case KEY:
                return values.get(0);
            case NUM_VALUE:
                return values.get(index);
            case TEXT_VALUE:
                return evenTextValue ? values.get(index) : "{T}";
            case TEXT_STATIC:
                return values.get(0);
            case SEPARATOR:
                return values.get(0);
            default:
                throw new IllegalStateException();
        }
    }

    public String groupKeyPart() {
        if (type == PathElementType.TEXT_STATIC) {
            return "{Y}";
        } else {
            return keyPart();
        }
    }

    public static String buildKey(List<PathElement> elements) {
        StringBuilder builder = new StringBuilder();

        for (PathElement element : elements) {
            builder.append(element.keyPart());
        }

        return builder.toString();
    }

    public static String buildGroupKey(List<PathElement> elements) {
        StringBuilder builder = new StringBuilder();

        for (PathElement element : elements) {
            builder.append(element.groupKeyPart());
        }

        return builder.toString();
    }

    public static String buildKeyWithConstant(List<PathElement> elements, int index, String replace) {
        StringBuilder builder = new StringBuilder();
        int counter = 0;

        for (PathElement element : elements) {
            if (counter++ == index) {
                builder.append(replace);
            } else {
                builder.append(element.keyPart());
            }
        }

        return builder.toString();
    }

    public static String buildKeySingleVariable(List<PathElement> elements, int urlIndex, int elementIndex, boolean textValueConstants) {
        StringBuilder builder = new StringBuilder();
        int counter = 0;

        for (PathElement element : elements) {
            if (counter++ == elementIndex) {
                builder.append(element.groupKeyPart());
            } else {
                builder.append(element.constantKeyPart(urlIndex, textValueConstants));
            }
        }

        return builder.toString();
    }

    public static List<PathElement> buildPathSingleVariableNumber(List<PathElement> elements, int urlIndex, int elementIndex) {
        List<PathElement> newElements = new ArrayList<>();
        int counter = 0;

        for (PathElement element : elements) {
            switch (element.type) {
                case DOMAIN:
                case KEY:
                case TEXT_STATIC:
                case SEPARATOR:
                    newElements.add(element);
                    break;
                case NUM_VALUE:
                    PathElementType newType = elementIndex == counter ? PathElementType.NUM_VALUE : PathElementType.TEXT_STATIC;
                    newElements.add(new PathElement(newType, element.isGetPart, element.values.get(urlIndex)));
                    break;
                case TEXT_VALUE:
                    newElements.add(new PathElement(PathElementType.TEXT_VALUE, element.isGetPart, element.values.get(urlIndex)));
                    break;
                default:
                    throw new IllegalStateException();
            }

            counter++;
        }

        return newElements;
    }
}
