package ut.maxorato.bsc.crawler.model;

import java.util.ArrayList;
import java.util.List;

public class TemplateResult {
    public final UrlTemplate template;
    public final List<String> generatedUrls;
    public final List<String> failedUrls;
    public final boolean recordFailedUrls;
    public int totalAttempts;

    public TemplateResult(UrlTemplate template, boolean recordFailedUrls) {
        this.template = template;
        this.generatedUrls = new ArrayList<>();
        this.failedUrls = new ArrayList<>();
        this.recordFailedUrls = recordFailedUrls;
    }

    public void record(String url, boolean success) {
        if (success) {
            generatedUrls.add(url);
        } else if (recordFailedUrls) {
            failedUrls.add(url);
        }
        totalAttempts++;
    }

    public float getRatio() {
        return totalAttempts > 0 ? (float)generatedUrls.size() / (float)totalAttempts : 1;
    }

    public boolean isFailure(int minimumCountThreshold, float ratioThreshold) {
        return totalAttempts >= minimumCountThreshold && getRatio() < ratioThreshold;
    }
}
